@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.71 2004/02/25 00:10:59 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.82 2004/07/27 22:06:31 richb Exp $
  *
- *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,11 +21,13 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include "calctool.h"
 #include <gdk/gdkkeysyms.h>
 
+#include "functions.h"
+
 time_t time();
 
 static void init_text();
@@ -34,19 +36,6 @@ double max_fix[MAXBASES] = {
     6.871947674e+10, 3.245185537e+32, 1.000000000e+36, 2.230074520e+43
 };
 
-double min_fix[MAXACC][MAXBASES] = {
-    { 2.500000000e-1, 3.750000000e-1, 1.000000000e-1,  4.375000000e-1 },
-    { 1.250000000e-1, 4.687500000e-2, 1.000000000e-2,  2.734375000e-2 },
-    { 6.250000000e-2, 5.859375000e-3, 1.000000000e-3,  1.708984375e-3 },
-    { 3.125000000e-2, 7.324218750e-4, 1.000000000e-4,  1.068115234e-4 },
-    { 1.562500000e-2, 9.155273437e-5, 1.000000000e-5,  6.675720215e-6 },
-    { 7.812500000e-3, 1.144409180e-5, 1.000000000e-6,  4.172325134e-7 },
-    { 6.906250000e-3, 1.430511475e-6, 1.000000000e-7,  2.607703209e-8 },
-    { 1.953125000e-3, 1.788139343e-7, 1.000000000e-8,  1.629814506e-9 },
-    { 9.765625000e-4, 2.235174179e-8, 1.000000000e-9,  1.018634066e-10 },
-    { 4.882812500e-4, 2.793967724e-9, 1.000000000e-10, 6.366462912e-12 }
-};
-
 char *base_str[]  = {          /* Strings for each base value. */
     N_("_Bin"), N_("_Oct"), N_("_Dec"), N_("He_x")
 };
@@ -60,7 +49,7 @@ char *base_desc[]  = {         /* Tooltips for each base value. */
 
 char *calc_res[] = {
     "accuracy", "base", "display", "mode", "showregisters", "trigtype",
-    "showzeroes", "showthousands"
+    "showzeroes", "showthousands", "syntax"
 };
 
 char *dtype_str[] = {          /* Strings for each display mode value. */
@@ -76,15 +65,11 @@ char *dtype_desc[] = {         /* Tooltips for each display mode value. */
 char *hyp_desc = N_("Set hyperbolic option for trigonometric functions");
 char *inv_desc = N_("Set inverse option for trigonometric functions");
 
-char *mode_str[]  = {          /* Strings for each mode value. */
-    N_("BASIC"), N_("FINANCIAL"), N_("SCIENTIFIC")
-};
-
 char *mstrs[] = {              /* Mode titles to be added to the titlebar. */
-    N_("Basic Mode"), N_("Financial Mode"), N_("Scientific Mode")
+    N_("Basic Mode"), N_("Financial Mode"), 
+    N_("Scientific Mode"), N_("Expression Mode")
 };
 
-
 char *ttype_str[] = {          /* Strings for each trig type value. */
     N_("De_grees"), N_("Gr_adians"), N_("_Radians")
 };
@@ -103,8 +88,11 @@ int basevals[4] = { 2, 8, 10, 16 };
 
 char *Rbstr[MAXBASES]     = { "BIN", "OCT", "DEC", "HEX" };
 char *Rdstr[MAXDISPMODES] = { "ENG", "FIX", "SCI" };
-char *Rmstr[MAXMODES]     = { "BASIC", "FINANCIAL", "SCIENTIFIC" };
+char *Rmstr[MAXMODES]     = { "BASIC", "FINANCIAL", 
+                              "SCIENTIFIC", "SCIENTIFIC_EXP" };
 char *Rtstr[MAXTRIGMODES] = { "DEG", "GRAD", "RAD" };
+char *Rsstr[MAXSYNTAX]    = { "ARITHMETIC", "ARITHMETIC_PRECEDENCE" };
+
 
 /* Valid keys when an error condition has occured. */
 /*                           Clr */
@@ -145,7 +133,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_7, GDK_7,          GDK_KP_7, GDK_KP_Home, GDK_R7, 0 },
     '7',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("8"),
@@ -155,7 +145,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_8, GDK_8,          GDK_KP_8, GDK_KP_Up, 0 },
     '8',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("9"),
@@ -165,7 +157,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_9, GDK_9,          GDK_KP_9, GDK_KP_Page_Up, GDK_R9, 0 },
     '9',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {    
     N_("/"),
@@ -175,27 +169,33 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_slash, GDK_slash,      GDK_KP_Divide, GDK_R5, GDK_slash,      0 },
     '/',
     M_NONE,
-    do_calc
+    do_calc,
+    N_("/"),
+    binop
 },
 {
-    "    ",
-    "    ",
-    NULL,
-    { 0, 0 },
-    { 0, 0 },
-    ' ',
+    N_("("),
+    N_("Start group of calculations"),
+    N_("Left bracket"),
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_parenleft, 0 },
+    '(',
     M_NONE,
-    do_none
+    do_paren,
+    NULL,
+    parenthesis
 },
 {
-    N_("Bsp"),
+    N_("Bksp"),
     N_("Remove rightmost character from displayed value"),
     N_("Backspace"),
     { 0, 0 },
     { GDK_BackSpace, 0 },
     '\010',
     M_NONE,
-    do_delete
+    do_delete,
+    NULL,
+    bsp
 },
 {
     N_("CE"),
@@ -205,7 +205,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_BackSpace,    GDK_Escape, 0 },
     '\013',
     M_NONE,
-    do_clear_entry
+    do_clear_entry,
+    NULL,
+    clear
 },
 {
     N_("Clr"),
@@ -215,7 +217,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_Delete, 0 },
     '\177',
     M_NONE,
-    do_clear
+    do_clear,
+    NULL,
+    clear
 },
 
 /* Row 2. */
@@ -227,7 +231,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_4, GDK_4,          GDK_KP_4, GDK_KP_Left, 0 },
     '4',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("5"),
@@ -237,7 +243,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_5, GDK_5,          GDK_KP_5, GDK_KP_Begin, GDK_R11, 0 },
     '5',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("6"),
@@ -247,7 +255,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_6, GDK_6,          GDK_KP_6, GDK_KP_Right, 0 },
     '6',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("*"),
@@ -257,47 +267,57 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_asterisk,   GDK_KP_Multiply, GDK_x, GDK_R6, 0 },
     '*',
     M_NONE,
-    do_calc
+    do_calc,
+    N_("*"),
+    binop
 },
 {
-    "    ",
-    "    ",
-    NULL,
-    { 0, 0 },
-    { 0, 0 },
-    ' ',
+    N_(")"),
+    N_("End group of calculations"),
+    N_("Right bracket"),
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_parenright, 0 },
+    ')',
     M_NONE,
-    do_none
+    do_paren,
+    NULL,
+    parenthesis
 },
 {
     N_("+/-"),
-    N_("Change sign"),
+    N_("Change sign [c]"),
     NULL,
     { 0,     0 },
     { GDK_c, 0 },
     'c',
     M_NONE,
-    do_immed
+    do_immed, 
+    N_("Chs"),
+    neg
 },
 {
     N_("Int"),
-    N_("Integer portion of displayed value"),
+    N_("Integer portion of displayed value [i]"),
     N_("Integer portion"),
     { 0, 0 },
     { GDK_i, 0 },
     'i',
     M_NONE,
-    do_portion
+    do_portion,
+    NULL,
+    func
 },
 {
     N_("Sto"),
-    N_("Store displayed value in memory register"),
+    N_("Store displayed value in memory register [S]"),
     N_("Store to register"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_S, 0 },
     'S',
     M_STO,
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 
 /* Row 3. */
@@ -309,7 +329,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_1, GDK_1,          GDK_KP_1, GDK_KP_End, GDK_R13, 0 },
     '1',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("2"),
@@ -319,7 +341,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_2, GDK_2,          GDK_KP_2, GDK_KP_Down, 0 },
     '2',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },    
 {     
     N_("3"),
@@ -329,7 +353,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_3, GDK_3,          GDK_KP_3, GDK_KP_Page_Down, GDK_R15, 0 },
     '3',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {
     N_("-"),
@@ -339,7 +365,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_minus, GDK_KP_Subtract, GDK_R4, 0 },
     '-',
     M_NONE,
-    do_calc
+    do_calc,
+    NULL,
+    unop | binop
 },
 {
     N_("%"),
@@ -349,37 +377,45 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_percent, 0 },
     '%',
     M_NONE,
-    do_calc
+    do_immed,  
+    NULL,
+    immediate
 },
 {
     N_("Sqrt"),
-    N_("Square root"),
+    N_("Square root [s]"),
     NULL,
     { 0, 0 },   
     { GDK_s, 0 },
     's',
     M_NONE,
-    do_immed
+    do_immed, 
+    NULL,
+    func
 },
 {
     N_("Frac"),
-    N_("Fractional portion of displayed value"),
+    N_("Fractional portion of displayed value [:]"),
     N_("Fractional portion"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_colon, 0 },
     ':',
     M_NONE,
-    do_portion
+    do_portion,
+    NULL,
+    func  
 },
 {
     N_("Rcl"),
-    N_("Retrieve memory register to display"),
+    N_("Retrieve memory register to display [R]"),
     N_("Retrieve from register"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_R, 0 },
     'R',
     M_RCL,
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 
 /* Row 4. */
@@ -391,7 +427,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_0, GDK_0,          GDK_KP_0, GDK_KP_Insert, 0 },
     '0',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    number
 },
 {    
     N_("."),
@@ -401,7 +439,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_period, GDK_KP_Decimal, GDK_KP_Delete, GDK_KP_Separator, 0 },
     '.',
     M_NONE,
-    do_point
+    do_point,
+    NULL,
+    number
 },
 {
     N_("="),
@@ -411,7 +451,9 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_equal, GDK_KP_Enter, GDK_Return, GDK_equal,      0 },
     '=',
     M_NONE,
-    do_calc
+    do_calc,
+    NULL,
+    enter
 },
 {
     N_("+"),
@@ -421,47 +463,57 @@ struct button b_buttons[B_NOBUTTONS] = {   /* Basic mode button values. */
     { GDK_plus,       GDK_plus, GDK_KP_Add, 0 },
     '+',
     M_NONE,
-    do_calc
+    do_calc,
+    NULL,
+    binop
 },
 {
-    N_("1/<i>x</i>"),
-    N_("Reciprocal"),
+  N_("1/<i>x</i>"),
+    N_("Reciprocal [r]"),
     NULL,
     { 0, 0 },
     { GDK_r, 0 },
     'r',
     M_NONE,
-    do_immed
+    do_immed, 
+    N_("Recip"),
+    inv
 },
 {
     N_("<i>x</i><sup>2</sup>"),
-    N_("Square"),
+    N_("Square [@]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_at, 0 },
     '@',
     M_NONE,
-    do_immed
+    do_immed,  
+    N_("^2"),
+    immediate | postfixop
 },
 {
     N_("Abs"),
-    N_("Absolute value"),
+    N_("Absolute value [u]"),
     NULL,
     { 0, 0 },
     { GDK_u, 0 },
     'u',
     M_NONE,
-    do_portion
+    do_portion,
+    NULL,
+    func
 },
 {
     N_("Exch"),
-    N_("Exchange displayed value with memory register"),
+    N_("Exchange displayed value with memory register [X]"),
     N_("Exchange with register"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_X, 0 },
     'X',
     M_EXCH,
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 };
 
@@ -481,95 +533,113 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
 
 {
     N_("Ctrm"),
-    N_("Compounding term"),
+    N_("Compounding term [m]"),
     NULL,
     { 0, 0 },
     { GDK_m, 0 },
     'm',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 {
     N_("Ddb"),
-    N_("Double-declining depreciation"),
+    N_("Double-declining depreciation [d]"),
     NULL,
     { 0,     0 },
     { GDK_d, 0 },
     'd',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 {
     N_("Fv"),
-    N_("Future value"),
+    N_("Future value [v]"),
     NULL,
     { 0, 0 },
     { GDK_v, 0 },
     'v',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 {
     N_("Pmt"),
-    N_("Periodic payment"),
+    N_("Periodic payment [P]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_P, 0 },
     'P',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 {
     N_("Pv"),
-    N_("Present value"),
+    N_("Present value [p]"),
     NULL,
     { 0, 0 },
     { GDK_p, 0 },
     'p',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 {
     N_("Rate"),
-    N_("Periodic interest rate"),
+    N_("Periodic interest rate [T]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_T, 0 },
     'T',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 {
     N_("Sln"),
-    N_("Straight-line depreciation"),
+    N_("Straight-line depreciation [l]"),
     NULL,
     { 0, 0 },
     { GDK_l, 0 },
     'l',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 { 
     N_("Syd"),
-    N_("Sum-of-the years'-digits depreciation"),
+    N_("Sum-of-the years'-digits depreciation [Y]"),
     NULL,
     { 0, 0 },
     { GDK_Y, 0 },
     'Y',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 
 /* Row 2. */
 { 
     N_("Term"),
-    N_("Payment period"),
+    N_("Payment period [T]"),
     NULL,
     { 0, 0 },
     { GDK_T, 0 },
     'T',
     M_NONE,
-    do_business
+    do_business,
+    NULL,
+    none
 },
 { 
     "    ",
@@ -579,7 +649,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 { 
     "    ",
@@ -589,7 +661,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 { 
     "    ",
@@ -599,7 +673,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 { 
     "    ",
@@ -609,7 +685,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 { 
     "    ",
@@ -619,7 +697,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 { 
     "    ",
@@ -629,7 +709,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 {
     "    ",
@@ -639,7 +721,9 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 };
 
@@ -665,7 +749,9 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_less, 0 },
     '<',
     M_LSHF,
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 {
     N_(">"),
@@ -675,27 +761,33 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_greater, 0 },
     '>',
     M_RSHF,   
-    do_pending
+    do_pending,
+    NULL,
+    none
 },            
 {             
     N_("&amp;16"),
-    N_("16-bit unsigned integer value of display"),
+    N_("16-bit unsigned integer value of display (])"),
     N_("16 bit unsigned integer"),
     { 0, 0 },        
     { GDK_bracketright, 0 },
     ']',
     M_NONE,   
-    do_immed  
+    do_immed,  
+    N_("u16"),
+    func
 },            
 {             
     N_("&amp;32"),
-    N_("32-bit unsigned integer value of display"),
+    N_("32-bit unsigned integer value of display ([)"),
     N_("32 bit unsigned integer"),
     { 0, 0 },
     { GDK_bracketleft, 0 },
     '[',
     M_NONE,
-    do_immed
+    do_immed,  
+    N_("u32"),
+    func
 },
 {
     "    ",
@@ -705,119 +797,143 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { 0, 0 },
     ' ',
     M_NONE,
-    do_none
+    do_none,
+    NULL,
+    none
 },
 {
-    N_("("),
-    N_("Start group of calculations"),
-    N_("Left bracket"),
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_parenleft, 0 },
-    '(',
+    "    ",
+    "    ",
+    NULL,
+    { 0, 0 },
+    { 0, 0 },
+    ' ',
     M_NONE,
-    do_paren
+    do_none,
+    NULL,
+    none
 },
 {
-    N_(")"),
-    N_("End group of calculations"),
-    N_("Right bracket"),
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_parenright, 0 },
-    ')',
+    "    ",
+    "    ",
+    NULL,
+    { 0, 0 },
+    { 0, 0 },
+    ' ',
     M_NONE,
-    do_paren
+    do_none,
+    NULL,
+    none
 },
 {
     N_("Acc"),
-    N_("Set accuracy from 0 to 9 numeric places"),
+    N_("Set accuracy from 0 to 9 numeric places [a]"),
     N_("Accuracy"),
     { 0,     0 },
     { GDK_a, 0 },
     'a',
     M_ACC,
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 
 /* Row 2. */
 {
     N_("Con"),
-    N_("Constants"),
+    N_("Constants [#]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },   
     { GDK_numbersign, 0 },   
     '#',
     M_CON,            
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 {
     N_("Fun"),
-    N_("User-defined functions"),
+    N_("User-defined functions [f]"),
     NULL,
     { 0,     0 },
     { GDK_f, 0 },
     'f',
     M_FUN,
-    do_pending
+    do_pending,
+    NULL,
+    none
 },
 {
     N_("Exp"),
-    N_("Enter an exponential number"),
+    N_("Enter an exponential number [e]"),
     N_("Exponential"),
     { 0,     0 },
     { GDK_e, 0 },
     'e',
     M_NONE,
-    do_expno
+    do_expno,
+    N_("*10^"),
+    expnum
 },
 {
     N_("e<sup><i>x</i></sup>"),
-    N_("e to the power of displayed value"),
+    N_("e to the power of displayed value [{]"),
     N_("E to the x"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_braceleft, 0 },
     '{',
     M_NONE,
-    do_immed
+    do_immed, 
+    N_("e^"),
+    immediate | prefixop
 },
 {
     N_("10<sup><i>x</i></sup>"),
-    N_("10 to the power of displayed value"),
+    N_("10 to the power of displayed value [}]"),
     N_("Ten to the x"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_braceright, 0 },
     '}',
     M_NONE,
-    do_immed
+    do_immed, 
+    N_("10^"),
+    prefixop
 },       
 {        
     N_("<i>x</i><sup><i>y</i></sup>"),
-    N_("Raise displayed value to the power of y"),
+    N_("Raise displayed value to the power of y [y]"),
     N_("X to the y"),
     { 0, 0 },   
     { GDK_y, 0 },
     'y',
     M_NONE,
-    do_calc
+    do_calc,
+    N_("^"),
+    binop | postfixop
 },       
 {        
     N_("<i>x</i>!"),
-    N_("Factorial of displayed value"),
+    N_("Factorial of displayed value [!]"),
     N_("Factorial"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_exclam, 0 },
     '!',
     M_NONE,
-    do_immed
+    do_immed, 
+    N_("!"),
+    immediate | postfixop
 },
 {
     N_("Rand"),
-    N_("Random number in the range 0.0 to 1.0"),
+    N_("Random number in the range 0.0 to 1.0 [?]"),
     N_("Random number"),
     { GDK_SHIFT_MASK, 0 },
     { GDK_question, 0 },
     '?',
     M_NONE,
-    do_immed
+    do_immed, 
+    NULL,
+    none
 },
 
 /* Row 3. */
@@ -829,7 +945,9 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_D,          0 },
     'D',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    none
 },
 {
     N_("E"),
@@ -839,7 +957,9 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_E,          0 },
     'E',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    none
 },
 {
     N_("F"),
@@ -849,57 +969,69 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_F,          0 },
     'F',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    none
 },
 {
     N_("Cos"),
-    N_("Cosine"),
+    N_("Cosine [J]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_J, 0 },
     'J',
     M_NONE,
-    do_trig
+    do_trig,
+    NULL,
+    func
 },
 {
     N_("Sin"),
-    N_("Sine"),
+    N_("Sine [K]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_K, 0 },
     'K',
     M_NONE,
-    do_trig
+    do_trig,
+    NULL,
+    func
 },
 {        
     N_("Tan"),
-    N_("Tangent"),
+    N_("Tangent [L]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_L, 0 },
     'L',
     M_NONE,
-    do_trig
+    do_trig,
+    NULL,  
+    func
 },     
 {      
     N_("Ln"),
-    N_("Natural log"),
+    N_("Natural log [N]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_N, 0 },
     'N',
     M_NONE,
-    do_immed
+    do_immed, 
+    NULL,
+    func
 },
 { 
     N_("Log"),
-    N_("Base 10 log"),
+    N_("Base 10 log [G]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_G, 0 },
     'G',
     M_NONE,
-    do_immed
+    do_immed, 
+    NULL,
+    func
 },
 
 /* Row 4. */
@@ -911,7 +1043,9 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_A,          0 },
     'A',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    none
 },
 {
     N_("B"),
@@ -921,7 +1055,8 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_B,          0 },
     'B',
     M_NONE,
-    do_number
+    do_number,
+    NULL
 },    
 {     
     N_("C"),
@@ -931,67 +1066,80 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     { GDK_C,          0 },
     'C',
     M_NONE,
-    do_number
+    do_number,
+    NULL,
+    none
 },
 {
     N_("Or"),
-    N_("Logical OR"),
-    "logical OR",
+    N_("Bitwise OR"),
+    "bitwise OR [!]",
     { GDK_SHIFT_MASK, 0 },
     { GDK_bar, 0 },
     '|',
     M_NONE,
-    do_calc
+    do_calc,
+    N_(" Or "),
+    binop
 },
 {
     N_("And"),
-    N_("Logical AND"),
+    N_("Bitwise AND [&]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_ampersand, 0 },
     '&',
     M_NONE,
-    do_calc
+    do_calc,
+    N_(" And "),
+    binop
 },       
 {        
     N_("Not"),
-    N_("Logical NOT"),
+    N_("Bitwise NOT [~]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
     { GDK_asciitilde, 0 },
     '~',
     M_NONE,
-    do_immed
+    do_immed, 
+    N_("~"),
+    unop | immediate
 },
 {
     N_("Xor"),
-    N_("Logical XOR"),
+    N_("Bitwise XOR [^]"),
     NULL,
     { GDK_SHIFT_MASK, GDK_SHIFT_MASK,  0 },
     { GDK_caret,      GDK_asciicircum, 0 },
     '^',
     M_NONE,
-    do_calc
+    do_calc,
+    N_(" Xor "),
+    binop
 },
 {
     N_("Xnor"),
-    N_("Logical XNOR"),
+    N_("Bitwise XNOR [n]"),
     NULL,
     { 0, 0 },
     { GDK_n, 0 },
     'n',
     M_NONE,
-    do_calc
+    do_calc,
+    N_(" Xnor "),
+    binop
 },
 };
 
-
 void
 do_calctool(int argc, char **argv)
 {
     char *ptr, title[MAXLINE];
     int i;
 
+    build_word_map();          /* Initialize i18n. */
+
     v->progname = argv[0];     /* Save programs name. */
     v->appname  = NULL;
 
@@ -1049,10 +1197,11 @@ do_calctool(int argc, char **argv)
         }
     }
 
-    SPRINTF(title, "%s- %s", v->tool_label, _(mstrs[(int) v->modetype]));
+    SPRINTF(title, "%s - %s", v->tool_label, _(mstrs[(int) v->modetype]));
     set_title(FCP_KEY, title);
 
     show_display(v->MPdisp_val);     /* Output in correct display mode. */
+
     start_tool();                    /* Display the calculator. */
 }
 
@@ -1066,8 +1215,8 @@ doerr(char *errmes)
         return;
     }
     STRCPY(v->display, errmes);
-    set_display(v->display);
     set_error_state(TRUE);
+    set_display(v->display, FALSE);
     beep();
 }
 
@@ -1075,16 +1224,18 @@ doerr(char *errmes)
 static void
 init_text()         /* Setup constant strings. */
 {
-    STRCPY(v->con_names[0], _("kilometers per hour or miles per hour"));
+    STRCPY(v->con_names[0], _("Kilometer-to-mile conversion factor"));
     STRCPY(v->con_names[1], _("square root of 2"));
     STRCPY(v->con_names[2], _("e"));
     STRCPY(v->con_names[3], _("pi"));
-    STRCPY(v->con_names[4], _("centimeters or inches"));
+    STRCPY(v->con_names[4], _("Centimeter-to-inch conversion factor"));
     STRCPY(v->con_names[5], _("degrees in a radian"));
     STRCPY(v->con_names[6], _("2 ^ 20"));
-    STRCPY(v->con_names[7], _("grams or ounces"));
-    STRCPY(v->con_names[8], _("kilojoules or British thermal units"));
-    STRCPY(v->con_names[9], _("cubic centimeters or cubic inches"));
+    STRCPY(v->con_names[7], _("Gram-to-ounce conversion factor"));
+    STRCPY(v->con_names[8], 
+           _("Kilojoule-to-British-thermal-unit conversion factor"));
+    STRCPY(v->con_names[9], 
+           _("Cubic-centimeter-to-cubic-inch conversion factor"));
 }
 
 
