@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.50 2004/02/18 16:22:18 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.62 2004/10/15 19:20:48 sampie Exp $
  *
- *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
  *  02111-1307, USA.
  */
 
+#ifndef CALCTOOL_H
+#define CALCTOOL_H
+
 #include "config.h"
 
 #include <string.h>
@@ -46,8 +49,9 @@
 #define UNLINK       (void) unlink
 
 /* Menu bar menu types. */
+
 enum mb_type { M_ABOUT, M_ASCII, M_BASIC, M_CONTENTS, M_COPY,  M_FIN,
-               M_PASTE, M_QUIT,  M_REGS,  M_SCI,      M_TSEP,  M_ZEROES };
+	       M_PASTE, M_QUIT,  M_REGS,  M_SCI, M_EXP, M_TSEP, M_ZEROES };
 
 enum base_type { BIN, OCT, DEC, HEX };      /* Base definitions. */
 
@@ -69,17 +73,21 @@ enum num_type { ENG, FIX, SCI };            /* Number display mode. */
 
 /* Resources. */
 enum res_type { R_ACCURACY, R_BASE, R_DISPLAY, R_MODE, R_REGS, R_TRIG,
-                R_ZEROES,   R_TSEP
+                R_ZEROES,   R_TSEP, R_SYNTAX
 };
 
 enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 
+enum trig_func { SIN=1, COS=2, TAN=4 };
+
 /* Abbreviations for the gcalctool keyboard and menu equivalents. */
 
 #define KEY_DIV   b_buttons[3]              /* / */
+#define KEY_LPAR  b_buttons[4]              /* ( */
 #define KEY_BSP   b_buttons[5]              /* Control-h */
 
 #define KEY_MUL   b_buttons[11]             /* * */
+#define KEY_RPAR  b_buttons[12]             /* ) */
 
 #define KEY_CHS   b_buttons[13]             /* C */
 #define KEY_INT   b_buttons[14]             /* i */
@@ -112,8 +120,6 @@ enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 #define KEY_RSFT  s_buttons[1]              /* > */
 #define KEY_16    s_buttons[2]              /* [ */
 #define KEY_32    s_buttons[3]              /* ] */
-#define KEY_LPAR  s_buttons[5]              /* ( */
-#define KEY_RPAR  s_buttons[6]              /* ) */
 #define KEY_ACC   s_buttons[7]              /* A */
 
 #define KEY_CON   s_buttons[8]              /* # */
@@ -145,7 +151,7 @@ enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 #define SCOLS          8      /* No of columns of Scientific Mode buttons. */
 #define SROWS          4      /* No of rows of Scientific Mode buttons. */
 
-#define EQUAL(a, b)    !strncmp(a, b, strlen(b))
+#define EQUAL(a, b)    (strlen(a)==strlen(b)) & !strcmp(a, b) 
 
 #define INC            { argc--; argv++; }
 #define IS_KEY(v, n)   (v == n)
@@ -159,7 +165,7 @@ enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 #endif /*LINT_CAST*/
 
 #define MAX_DIGITS     40         /* Maximum displayable number of digits. */
-#define MAX_BUFFER     (MAX_DIGITS + MAX_DIGITS / 3 * MB_LEN_MAX + 1)
+#define MAX_LOCALIZED  (MAX_DIGITS * (1 + MB_LEN_MAX) + MB_LEN_MAX)
 
 /* Maximum number of various graphics pieces. */
 #define MAXITEMS       8          /* Maximum number of panel items. */
@@ -169,16 +175,17 @@ enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 #define MAXLINE        256        /* Length of character strings. */
 #endif /*MAXLINE*/
 
-#define MAXACC         10    /* Max. number of digits+1 after numeric point. */
+#define MAXACC         30    /* Max. number of digits after numeric point. */
 #define MAXBASES       4          /* Maximum number of numeric bases. */
 #define MAXCONFUN      10         /* Maximum number of constants/functions. */
 #define MAXDISPMODES   3          /* Maximum number of display modes. */
 #define MAXEXTRAS      8          /* Maximum number of keysym alternates. */
-#define MAXMODES       3          /* Maximum number of calculator modes. */
+#define MAXMODES       4          /* Maximum number of calculator modes. */
 #define MAXREGS        10         /* Maximum number of memory registers. */
 #define MAXSTACK       256        /* Parenthese stack size. */
 #define MAXTRIGMODES   3          /* Maximum number of trig. modes. */
 #define MAXVKEYS       1          /* Number of valid keys after an error. */
+#define MAXSYNTAX      2          /* Number of syntaxes in calculator */
 
 #define MCOLS          8          /* Number of columns of "special" keys. */
 #define MROWS          2          /* Number of rows of "special" keys. */
@@ -208,8 +215,50 @@ enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 #define FALSE          0
 #endif /*FALSE*/
 
+
 typedef unsigned long  BOOLEAN;
 
+enum button_flags {
+    none         = 0,          /* No flags */
+    binop        = 1,          /* Is binary operation ie. A op B */ 
+    unop         = 2,          /* Is unary operation ie. op A. */
+    enter        = 4,          /* Expression is entered */
+    number       = 8,          /* Number button */
+    immediate    = 16,         /* Button does immediate function */
+
+    parenthesis  = 64,         /* Parenthesis */
+    func         = 128,        /* Function */
+    bsp          = 256,        /* Backspace */
+    clear        = 512,        /* Clear display */
+    neg          = 1024,       /* Negate display */
+    inv          = 2048,       /* Reciprocial */
+    con          = 4096,       /* Constant */
+    regrcl       = 8192,       /* Recall register */
+    expnum       = 16384,      /* Exponential number */
+    postfixop    = 32768,      /* Unary postfix operation */
+    prefixop     = 65536       /* Unary prefix operation */
+};
+
+struct exprm_state {       /* Expression mode state */
+    int calc_complete : 1;
+    int numeric_ans : 1;    /* Show answer numerically 
+                             * (alternative is symbolically) 
+                             */
+    int ans[MP_SIZE];        /* Previously calculated answer */
+    int ansbak[MP_SIZE];     /* Pre ans */
+    char *expbak;
+};
+
+enum shiftd {
+    left = 0,
+    right 
+};
+
+enum syntax {
+    npa = 0,                 /* Non-precedence arithmetic */
+    exprs,                   /* Expression with arithmetic precedence */
+};
+
 struct button {
     char *str;               /* Button display string. */
     char *hstr;              /* Button help string. */
@@ -219,6 +268,8 @@ struct button {
     char func_char;          /* Unique function string character. */
     enum menu_type mtype;    /* Type of popup menu (if any). */
     void (*func)();          /* Function to obey on button press. */
+    char *symname;           /* Expression function name */
+    enum button_flags flags; /* Misc flags */
 };
 
 struct menu {
@@ -229,14 +280,16 @@ struct menu {
 };
 
 struct calcVars {                      /* Calctool variables and options. */
+    struct exprm_state e;               /* Expression mode state */
+  
     struct button *pending_but;        /* Button info. for pending op. */
     struct button *current;            /* Current button/character pressed. */
-
+  
     char *appname;                     /* Application name for resources. */
     char con_names[MAXREGS][MAXLINE];  /* Selectable constant names. */
     char display[MAXLINE];             /* Current calculator display. */
     char *exp_posn;                    /* Position of the exponent sign. */
-    char fnum[MAX_BUFFER];             /* Scratchpad for fixed numbers. */
+    char fnum[MAX_DIGITS];             /* Scratchpad for fixed numbers. */
     char fun_names[MAXREGS][MAXLINE];  /* Function names from .gcalctoolcf. */
     char fun_vals[MAXREGS][MAXLINE];   /* Function defs from .gcalctoolcf. */
     char *home;                        /* Pathname for users home directory. */
@@ -247,8 +300,7 @@ struct calcVars {                      /* Calctool variables and options. */
     char pstr[5];                      /* Current button text string. */
     const char *radix;                 /* Locale specific radix string. */
     char *shelf;                       /* PUT selection shelf contents. */
-    char snum[MAX_BUFFER];             /* Scratchpad for scientific numbers. */
-    char tnum[MAX_BUFFER];             /* Scratchpad for tsep numbers. */
+    char snum[MAX_DIGITS];             /* Scratchpad for scientific numbers. */
     const char *tsep;                /* Locale specific thousands seperator. */
     char *titleline;                   /* Value of titleline (if present). */
     char *tool_label;                  /* Title line for calculator window. */
@@ -257,10 +309,12 @@ struct calcVars {                      /* Calctool variables and options. */
     int MPdebug;                       /* If set, debug info. to stderr. */
     int MPerrors;                      /* If set, output errors to stderr. */
     int MPdisp_val[MP_SIZE];           /* Value of the current display. */
+    int MPexpr_val[MP_SIZE];           /* Value of the current expression. */
     int MPlast_input[MP_SIZE];         /* Previous number input by user. */
     int MPmvals[MAXREGS][MP_SIZE];     /* Memory register values. */
     int *MPnumstack[MAXSTACK];         /* Numeric stack for parens. */
     int MPresult[MP_SIZE];             /* Current calculator total value. */
+    int MPimresult[MP_SIZE];           /* Current intermediate result. */
     int MPtresults[3][MP_SIZE];        /* Current trigonometric results. */
 
     enum base_type base;            /* Current base: BIN, OCT, DEC or HEX. */
@@ -269,6 +323,10 @@ struct calcVars {                      /* Calctool variables and options. */
     enum num_type dtype;            /* Number display mode. */
     enum trig_type ttype;           /* Trig. type (deg, grad or rad). */
 
+    enum syntax syntax;             /* Calculation syntax mode */
+
+    char *expression;  /* Expression entered by user */
+  
     int accuracy;      /* Number of digits precision (Max 9). */
     int beep;          /* Indicates whether there is a beep sound on error. */
     int cur_ch;        /* Current character if keyboard event. */
@@ -297,10 +355,11 @@ struct calcVars {                      /* Calctool variables and options. */
     int show_zeroes;   /* Set if trailing zeroes should be shown. */
     int started;       /* Set just before window is displayed. */
     int toclear;       /* Indicates if display should be cleared. */
-} CalcVars;
+};
 
 typedef struct calcVars *Vars;
 
+
 /* MP definitions. */
 
 #define C_abs(x)    ((x) >= 0 ? (x) : -(x))
@@ -314,8 +373,10 @@ struct button *button_for_fc(char);
 struct button *button_for_value(int);
 struct button *copy_button_info(struct button *);
 
+void syntaxdep_show_display();
 char *button_str(int);
 char *convert(char *);
+char *gc_strdup(char *str);
 const char *get_radix();
 char *get_resource(enum res_type);
 const char *get_tsep();
@@ -327,18 +388,22 @@ unsigned short *get_but_data();
 int button_mods(int);
 int button_value(int);
 int char_val(char);
+int do_rcl_reg(int reg, int value[MP_SIZE]);
+int do_sto_reg(int reg, int value[MP_SIZE]);
+int do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc);
 int get_menu_entry(enum menu_type, int);
 int key_equal(struct button *, struct button);
 int main(int, char **);
 
-void adjust_radix(char *);
-void add_tsep();
 void beep();
+void build_word_map();
 void clear_display(int);
 void display_prop_sheet();
 void do_base(enum base_type);
 void do_business();
 void do_calc();
+void do_lr_calc();
+void do_expression();
 void do_calctool(int, char **);
 void do_clear();
 void do_clear_entry();
@@ -346,7 +411,7 @@ void do_delete();
 void do_numtype(enum num_type);
 void do_expno();
 void do_immed();
-void do_memory(int);
+void do_memory();
 void do_mode();
 void do_none();
 void do_number();
@@ -357,10 +422,10 @@ void do_portion();
 void do_trig();
 void do_trigtype(enum trig_type);
 void doerr(char *);
+void exp_del();
 void get_constant(int);
 void get_display();
 void get_function(int);
-void get_key_val(int *, char *);
 void get_options(int, char **);
 void grey_buttons(enum base_type);
 void handle_menu_selection(struct button *, int);
@@ -371,16 +436,16 @@ void init_frame_sizes();
 void init_vars();
 void init_Xvars(int *, char **);
 void key_init();
+void localize_number(char *, const char *);
 void load_resources();
 void make_frames();
 void make_menus();
 void make_reg(int, char *);
 void make_registers();
 void mperr();
-void MPstr_to_num(char *, enum base_type, gboolean, int *);
+void MPstr_to_num(char *, enum base_type, int *);
 void paren_disp(char);
 void process_item(struct button *);
-void process_stack(int, int, int);
 void process_str(char *);
 void push_num(int *);
 void push_op(int);
@@ -388,11 +453,14 @@ void put_resource(enum res_type, char *);
 void read_cfdefs();
 void read_resources();
 void read_str(char **, char *);
-void remove_tsep(char *);
+void refresh_display();
 void save_pending_values(struct button *);
 void save_resources();
+void set_accuracy_menu_item(int);
 void set_accuracy_toggle(int);
-void set_display(char *);
+void set_accuracy_tooltip(int);
+void set_display(char *, int);
+void write_display(char *);
 void set_error_state(int);
 void set_hyp_item(int);
 void set_ins_key();
@@ -404,7 +472,9 @@ void show_ascii_frame();
 void show_display(int *);
 void srand48();
 void start_tool();
+void str_replace(char **, char *, char *);
 void switch_hands(int);
+void update_statusbar(gchar *, const gchar *);
 void usage(char *);
 void win_display(enum fcp_type, int);
 
@@ -422,7 +492,6 @@ void mpasin(int *, int *);
 void mpatan(int *, int *);
 void mpcdm(double *, int *);
 void mpcim(int *, int *);
-void mpclr(int *, int *);
 void mpcmd(int *, double *);
 void mpcmf(int *, int *);
 void mpcmi(int *, int *);
@@ -446,3 +515,5 @@ void mpsqrt(int *, int *);
 void mpstr(int *, int *);
 void mpsub(int *, int *, int *);
 void mptanh(int *, int *);
+
+#endif /*CALCTOOL_H*/
