@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.133 2004/07/23 11:15:09 sampie Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.144 2004/10/15 19:20:48 sampie Exp $
  *
  *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -31,6 +31,8 @@
 #include "calctool.h"
 #include "extern.h"
 #include "dsdefs.h"
+#include "functions.h"
+#include "lr_parser.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
@@ -48,13 +50,13 @@
 #define BUT_8     X->bas_buttons[1]        /* 8 */
 #define BUT_9     X->bas_buttons[2]        /* 9 */
 #define BUT_CLR   X->bas_buttons[7]        /* Clr */
-#define BUT_ACC   X->sci_buttons[7]        /* A */
-#define BUT_A     X->sci_buttons[24]       /* a */
-#define BUT_B     X->sci_buttons[25]       /* b */
-#define BUT_C     X->sci_buttons[26]       /* c */
-#define BUT_D     X->sci_buttons[16]       /* d */
-#define BUT_E     X->sci_buttons[17]       /* e */
-#define BUT_F     X->sci_buttons[18]       /* f */
+#define BUT_ACC   X->sci_buttons[7]        /* a */
+#define BUT_A     X->sci_buttons[24]       /* A */
+#define BUT_B     X->sci_buttons[25]       /* B */
+#define BUT_C     X->sci_buttons[26]       /* C */
+#define BUT_D     X->sci_buttons[16]       /* D */
+#define BUT_E     X->sci_buttons[17]       /* E */
+#define BUT_F     X->sci_buttons[18]       /* F */
 
 struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkAccelGroup *kbd_accel;
@@ -84,6 +86,8 @@ struct Xobject {               /* Gtk+/Xlib graphics object. */
 
     GtkWidget *display_item;           /* Calculator display. */
     GtkWidget *rframe;                 /* Register window. */
+    GtkWidget *spframe;                /* Set Precision window. */
+    GtkWidget *spframe_val;
     GtkWidget *regs[MAXREGS];          /* Memory registers. */
     GtkWidget *menus[MAXMENUS];
     GtkWidget *trig[MAXTRIGMODES];     /* Trigonometric mode. */
@@ -133,7 +137,9 @@ static char *make_hostname(Display *);
 static gboolean aframe_key_cb(GtkWidget *, GdkEventKey *, gpointer);
 static gboolean dismiss_aframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean dismiss_rframe(GtkWidget *, GdkEvent *, gpointer);
+static gboolean dismiss_spframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean kframe_key_press_cb(GtkWidget *, GdkEventKey *, gpointer);
+static gboolean spframe_key_cb(GtkWidget *, GdkEventKey *, gpointer);
 
 static void about_cb(GtkAction *action);
 static void add_cf_column(GtkTreeView *, gchar *, gint, gboolean);
@@ -165,14 +171,15 @@ static void set_gcalctool_icon(void);
 static void set_memory_toggle(int);
 static void set_show_tsep_toggle(int);
 static void set_show_zeroes_toggle(int);
+static void show_precision_frame();
+static void spframe_cancel_cb(GtkButton *, gpointer);
+static void spframe_ok_cb(GtkButton *, gpointer);
 static void trig_cb(GtkToggleButton *, gpointer);
 static void ts_proc(GtkAction *action);
-static void gcalc_window_have_icons_notify (GConfClient *client,
-                                            guint cnxn_id,
-                                            GConfEntry *entry,
-                                            gpointer data);
-static void gcalc_window_get_menu_items (XVars X);
-static void gcalc_window_set_unset_image (gboolean have_icons);
+static void gcalc_window_have_icons_notify(GConfClient *, guint,
+                                            GConfEntry *, gpointer);
+static void gcalc_window_get_menu_items(XVars);
+static void gcalc_window_set_unset_image(gboolean);
 
 #define MENU_KEY_DIR "/desktop/gnome/interface"
 
@@ -200,7 +207,7 @@ static GtkActionEntry entries[] = {
     { "Contents", GTK_STOCK_HELP, N_("_Contents"), "F1",
       N_("Show help contents"), G_CALLBACK(mb_proc) },
     { "About", GNOME_STOCK_ABOUT, N_("_About"), NULL,
-      N_("Show about help"), G_CALLBACK(about_cb) },
+      N_("Show the About Gcalctool dialog"), G_CALLBACK(about_cb) },
 
     { "LSPlaces1",  NULL, N_("1 place"),   NULL, 
       N_("1 place"),   G_CALLBACK(mb_proc) },
@@ -281,26 +288,28 @@ static GtkToggleActionEntry toggle_entries[] = {
 static guint n_toggle_entries = G_N_ELEMENTS(toggle_entries);
 
 static GtkRadioActionEntry acc_radio_entries[] = {
-  { "SP0", NULL, N_("0 significant places"), NULL, 
+  { "SP0", NULL, N_("_0 significant places"), "<control>0", 
     N_("0 significant places"), '0' },
-  { "SP1", NULL, N_("1 significant place"),  NULL, 
+  { "SP1", NULL, N_("_1 significant place"),  "<control>1", 
     N_("1 significant place"),  '1' },
-  { "SP2", NULL, N_("2 significant places"), NULL,
+  { "SP2", NULL, N_("_2 significant places"), "<control>2",
     N_("2 significant places"), '2' },
-  { "SP3", NULL, N_("3 significant places"), NULL,
+  { "SP3", NULL, N_("_3 significant places"), "<control>3",
     N_("3 significant places"), '3' },
-  { "SP4", NULL, N_("4 significant places"), NULL,
+  { "SP4", NULL, N_("_4 significant places"), "<control>4",
     N_("4 significant places"), '4' },
-  { "SP5", NULL, N_("5 significant places"), NULL,
+  { "SP5", NULL, N_("_5 significant places"), "<control>5",
     N_("5 significant places"), '5' },
-  { "SP6", NULL, N_("6 significant places"), NULL,
+  { "SP6", NULL, N_("_6 significant places"), "<control>6",
     N_("6 significant places"), '6' },
-  { "SP7", NULL, N_("7 significant places"), NULL,
+  { "SP7", NULL, N_("_7 significant places"), "<control>7",
     N_("7 significant places"), '7' },
-  { "SP8", NULL, N_("8 significant places"), NULL,
+  { "SP8", NULL, N_("_8 significant places"), "<control>8",
     N_("8 significant places"), '8' },
-  { "SP9", NULL, N_("9 significant places"), NULL,
+  { "SP9", NULL, N_("_9 significant places"), "<control>9",
     N_("9 significant places"), '9' },
+  { "SPOther", NULL, N_("_Other (10) ..."), "<control>O",
+    N_("Set other precision"), 'O' },
 };
 static guint n_acc_radio_entries = G_N_ELEMENTS(acc_radio_entries);
 
@@ -355,6 +364,7 @@ static const gchar *ui_info =
 "    <menuitem action='SP7'/>"
 "    <menuitem action='SP8'/>"
 "    <menuitem action='SP9'/>"
+"    <menuitem action='SPOther'/>"
 "    <separator/>"
 "    <menuitem action='Show'/>"
 "  </popup>"
@@ -434,24 +444,26 @@ main(int argc, char **argv)
 
     do_calctool(argc, argv);
 
-    return 0;
+    return(0);
 }
 
 
 void 
 update_statusbar(gchar *text, const gchar *imagename)
 {
-  assert(text);
-  assert(imagename);
+    GtkImage *image = GTK_IMAGE(X->status_image);
 
-  GtkImage *image = GTK_IMAGE(X->status_image);
-  assert(image);
-  gtk_image_set_from_stock(image, imagename, GTK_ICON_SIZE_BUTTON);
-  gtk_statusbar_pop(GTK_STATUSBAR(X->statusbar), 0);
-  gtk_statusbar_push(GTK_STATUSBAR(X->statusbar), 0, text); 
+    assert(text);
+    assert(imagename);
+    assert(image);
+
+    gtk_image_set_from_stock(image, imagename, GTK_ICON_SIZE_BUTTON);
+    gtk_statusbar_pop(GTK_STATUSBAR(X->statusbar), 0);
+    gtk_statusbar_push(GTK_STATUSBAR(X->statusbar), 0, text); 
 }
 
 
+/*ARGSUSED*/
 static void
 about_cb(GtkAction *action)
 {
@@ -459,7 +471,8 @@ about_cb(GtkAction *action)
 
     if (about == NULL) {
         const gchar *authors[] = {
-            "Rich Burridge <rich.burridge@sun.com>\nSami Pietila <sampie@ariana-dsl.utu.fi>",
+            "Rich Burridge <rich.burridge@sun.com>",
+            "Sami Pietila <sampie@ariana-dsl.utu.fi>",
             NULL
         };
         const gchar *documenters[] = {
@@ -521,6 +534,7 @@ aframe_cancel_cb(GtkButton *button, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
 static gboolean
 aframe_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
@@ -549,6 +563,7 @@ aframe_ok_cb(GtkButton *button, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
 static void
 astz_proc(GtkAction *action)
 {
@@ -561,13 +576,14 @@ astz_proc(GtkAction *action)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi), v->show_zeroes);
         v->doing_mi = 0;
 
-        show_display(v->MPdisp_val);
+	syntaxdep_show_display();  //show_display(v->MPdisp_val);
 	put_resource(R_ZEROES, set_bool(v->show_zeroes == TRUE));
 	make_registers();
     }
 }
 
 
+/*ARGSUSED*/
 static void
 base_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -582,50 +598,6 @@ beep()
 }
 
 
-static GtkWidget *
-button_new_with_stock_image(const gchar *text, const gchar *stock_id)
-{
-    GtkWidget *button, *label, *image, *hbox, *align;
-    GtkStockItem item;
-
-    button = gtk_button_new();
-
-    if (GTK_BIN(button)->child) {
-        gtk_container_remove(GTK_CONTAINER(button), GTK_BIN(button)->child);
-    }
-
-    if (gtk_stock_lookup(stock_id, &item)) {
-        label = gtk_label_new_with_mnemonic(text);
-
-        gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(button));
-
-        image = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
-        hbox = gtk_hbox_new(FALSE, 2);
-
-        align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-
-        gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-        gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-        gtk_container_add(GTK_CONTAINER(button), align);
-        gtk_container_add(GTK_CONTAINER(align), hbox);
-        gtk_widget_show_all(align);
-
-        return(button);
-    }
-
-    label = gtk_label_new_with_mnemonic(text);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(button));
-
-    gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-
-    gtk_widget_show(label);
-    gtk_container_add(GTK_CONTAINER(button), label);
-
-    return(button);
-}
-
-
 /*ARGSUSED*/
 static void
 button_proc(GtkButton *widget, gpointer user_data)
@@ -705,6 +677,7 @@ cell_edited(GtkCellRendererText *cell, const gchar *path_string,
 }
 
 
+/*ARGSUSED*/
 static void
 cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
 {
@@ -966,6 +939,74 @@ create_cfframe(enum menu_type mtype, GtkWidget *dialog)
 }
 
 
+static void
+create_spframe()     /* Create auxiliary frame for Set Precision value. */
+{
+    GtkWidget *vbox, *hbox, *button_hbox, *label;
+    GtkWidget *set_button, *cancel_button;
+
+    X->spframe = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title(GTK_WINDOW(X->spframe), _("Set Precision"));
+    gtk_window_set_resizable(GTK_WINDOW(X->spframe), FALSE);
+    gtk_window_set_transient_for(GTK_WINDOW(X->spframe), GTK_WINDOW(X->kframe));
+    gtk_window_set_type_hint(GTK_WINDOW(X->spframe),
+                             GDK_WINDOW_TYPE_HINT_DIALOG);
+
+    vbox = gtk_vbox_new(FALSE, 12);
+    gtk_widget_show(vbox);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
+    gtk_container_add(GTK_CONTAINER(X->spframe), vbox);
+
+    hbox = gtk_hbox_new(FALSE, 6);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+    label = gtk_label_new_with_mnemonic(_("Significant _places:"));
+    gtk_widget_show(label);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+
+    X->spframe_val = gtk_spin_button_new_with_range(0, MAXACC, 1);
+    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(X->spframe_val),
+                                      GTK_UPDATE_IF_VALID);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(X->spframe_val), 
+                              (double) v->accuracy);
+    gtk_widget_show(X->spframe_val);
+    gtk_box_pack_start(GTK_BOX(hbox), X->spframe_val, FALSE, FALSE, 0);
+
+    button_hbox = gtk_hbutton_box_new();
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(button_hbox), GTK_BUTTONBOX_END);
+    gtk_widget_ref(button_hbox);
+    gtk_widget_show(button_hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), button_hbox, TRUE, TRUE, 0);
+    gtk_box_set_spacing(GTK_BOX(button_hbox), 12);
+
+    cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
+    gtk_widget_ref(cancel_button);
+    gtk_widget_show(cancel_button);
+    gtk_box_pack_start(GTK_BOX(button_hbox), cancel_button, FALSE, FALSE, 0);
+
+    set_button = gtk_button_new_with_mnemonic(_("_Set"));
+    g_return_if_fail(set_button != NULL);
+    gtk_widget_ref(set_button);
+    gtk_widget_show(set_button);
+    GTK_WIDGET_SET_FLAGS(set_button, GTK_CAN_DEFAULT);
+    gtk_window_set_default(GTK_WINDOW(X->spframe), set_button);
+    gtk_box_pack_start(GTK_BOX(button_hbox), set_button, FALSE, FALSE, 0);
+
+    g_signal_connect(G_OBJECT(X->spframe), "delete_event",
+                     G_CALLBACK(dismiss_spframe), NULL);
+    g_signal_connect(G_OBJECT(X->spframe), "key_press_event",
+                     G_CALLBACK(spframe_key_cb), NULL);
+    g_signal_connect(G_OBJECT(set_button), "clicked",
+                     G_CALLBACK(spframe_ok_cb), NULL);
+    g_signal_connect(G_OBJECT(cancel_button), "clicked",
+                     G_CALLBACK(spframe_cancel_cb), NULL);
+    gtk_widget_realize(X->spframe);
+}
+
+
 /* action_image[] is a list of Actions which are having icons associated.
  * We need to set/unset the icons if /desktop/gnome/interface/menus_have_icons
  * toggles.
@@ -997,6 +1038,7 @@ gcalc_window_get_menu_items(XVars X)
 }
 
 
+/*ARGSUSED*/
 static void
 gcalc_window_have_icons_notify(GConfClient *client, guint cnxn_id,
                                GConfEntry *entry, gpointer data)
@@ -1037,14 +1079,14 @@ gcalc_window_set_unset_image(gboolean have_icons)
 }
 
 
-void
+static void
 create_kframe()
 {
     char *hn;
-    int count;
     GError *error;
     GtkWidget *event_box, *view_widget;
     GtkTextBuffer *buffer;
+    GtkWidget *scrolledwindow;
 
     v->tool_label = NULL;
     if (v->titleline == NULL) {
@@ -1099,7 +1141,7 @@ create_kframe()
     gtk_box_pack_start(GTK_BOX(X->kvbox), X->menubar, FALSE, FALSE, 0);
 
 
-    GtkWidget *scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
+    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_show(scrolledwindow);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow), 
 				   GTK_POLICY_ALWAYS, 
@@ -1147,6 +1189,7 @@ create_kframe()
 
     X->sci_panel = make_but_panel(X->kvbox, X->sci_buttons,
                                   s_buttons, SROWS, SCOLS, "sci");
+    set_accuracy_tooltip(v->accuracy);
 
     gtk_widget_show(X->fin_panel);
     gtk_widget_show(X->mode_panel);
@@ -1339,7 +1382,7 @@ create_mode_panel(GtkWidget *main_vbox)
 }
 
 
-void
+static void
 create_rframe()
 {
     char line[MAXLINE];     /* Current memory register line. */
@@ -1376,6 +1419,7 @@ create_rframe()
 }
 
 
+/*ARGSUSED*/
 static void
 die_cb(GnomeClient *client, gpointer data)
 {
@@ -1405,6 +1449,16 @@ dismiss_rframe(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
+static gboolean
+dismiss_spframe(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+    X->spframe = NULL;
+
+    return(FALSE);
+}
+
+
 static void
 create_con_fun_menu(enum menu_type mtype)
 {
@@ -1466,6 +1520,7 @@ create_menu_item_with_markup(char *label, int menu_no, int user_data)
 }
 
 
+/*ARGSUSED*/
 static void
 disp_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -1494,7 +1549,7 @@ get_constant(int n)
 }
 
 
-void
+static void
 get_display()              /* The Copy function key has been pressed. */
 {
     gchar *string = NULL;
@@ -1544,12 +1599,9 @@ get_function(int n)
 static char *
 get_localized_numeric_point(void)
 {
-    struct lconv *locale_data;
     const char *decimal_point;
-    int decimal_point_len;
 
     decimal_point = localeconv()->decimal_point;
-    decimal_point_len = strlen(decimal_point);
 
     return(g_locale_to_utf8(decimal_point, -1, NULL, NULL, NULL));
 }
@@ -1578,7 +1630,27 @@ get_menu_entry(enum menu_type mtype, int offset)
 static void
 get_proc(GtkClipboard *clipboard, const gchar *text, gpointer data)
 {
-    process_str((char *) text);
+    switch (v->syntax) {
+        case npa: {
+            int ret = lr_parse((char *) text, v->MPdisp_val);
+
+            if (!ret) {
+                show_display(v->MPdisp_val);
+            } else {
+                update_statusbar(_("Clipboard contained malformed calculation"),
+                                 "gtk-dialog-error");
+            }
+            break;
+        }
+    
+        case exprs:
+            exp_append((char *) text);
+            refresh_display();
+            break;
+    
+        default:
+            assert(0);
+    }
 }
 
 
@@ -1618,7 +1690,7 @@ grey_buttons(enum base_type base)
 }
 
 
-void
+static void
 handle_selection()  /* Handle the GET function key being pressed. */
 {
     gtk_clipboard_request_text(gtk_clipboard_get(X->clipboard_atom),
@@ -1640,6 +1712,7 @@ help_cb()
 }
 
 
+/*ARGSUSED*/
 static void
 hyp_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -1647,6 +1720,7 @@ hyp_cb(GtkToggleButton *button, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
 static void
 inv_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -1661,7 +1735,7 @@ check_for_localized_numeric_point(int keyval)
     gunichar ch;
 
     ch = gdk_keyval_to_unicode(keyval);
-    g_return_if_fail(g_unichar_validate(ch));
+    g_return_val_if_fail(g_unichar_validate(ch), FALSE);
 
     outbuf[g_unichar_to_utf8(ch, outbuf)] = '\0';
 
@@ -1692,6 +1766,7 @@ check_vals(int n, int keyval, int state,
     return(FALSE);
 }
 
+/*ARGSUSED*/
 static gboolean
 kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
@@ -1745,7 +1820,6 @@ void
 make_frames()
 {
     struct button *n;
-    GtkWidget *menu;
 
     X->clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
     create_kframe();                     /* Create main gcalctool window. */
@@ -1753,7 +1827,7 @@ make_frames()
     set_mode(v->modetype);
 
     n = (struct button *) g_object_get_data(G_OBJECT(BUT_ACC), "button");
-    menu = create_menu(n->mtype, n);
+    (void) create_menu(n->mtype, n);
     set_accuracy_toggle(v->accuracy);
     set_show_tsep_toggle(v->show_tsep);
     set_show_zeroes_toggle(v->show_zeroes);
@@ -1805,6 +1879,28 @@ set_accessible_name(GtkWidget *widget, struct button button)
 }
 
 
+void
+set_accuracy_menu_item(int accuracy)
+{
+    char label[MAXLINE];
+
+    SPRINTF(label, _("Other (%d) ..."), accuracy);
+    g_object_set(gtk_ui_manager_get_action(X->ui, "/AccMenu/SPOther"),
+                 "label", label, NULL);
+}
+
+
+void
+set_accuracy_tooltip(int accuracy)
+{
+    char tooltip[MAXLINE];
+
+    SPRINTF(tooltip, _("Set accuracy from 0 to %d numeric places. Currently set to %d places. [a]"), 
+            MAXACC, accuracy);
+    gtk_tooltips_set_tip(X->tips, BUT_ACC, tooltip, "");
+}
+
+
 static GtkWidget *
 make_but_panel(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                struct button buttons[], int rows, int cols, char *tag)
@@ -1875,8 +1971,7 @@ make_reg(int n, char *str)
 static GtkWidget *
 create_menu(enum menu_type mtype, struct button *n)
 {
-    int count = 0;
-    int m     = (int) mtype;
+    int m = (int) mtype;
 
     switch (mtype) {
         case M_EXCH :
@@ -1897,6 +1992,7 @@ create_menu(enum menu_type mtype, struct button *n)
     if (X->menus[m] == NULL) {
         if (mtype == M_ACC) {
             X->menus[m] = gtk_ui_manager_get_widget(X->ui, "/AccMenu");
+            set_accuracy_menu_item(v->accuracy);
         } else if (mtype == M_LSHF) {
             X->menus[m] = gtk_ui_manager_get_widget(X->ui, "/LeftShiftMenu");
         } else if (mtype == M_RSHF) {
@@ -1942,6 +2038,7 @@ menu_button_button_press_cb(GtkWidget *widget,
 }
 
 
+/*ARGSUSED*/
 static gboolean
 menu_button_key_press_cb(GtkWidget *widget, 
                          GdkEventKey *event, gpointer data)
@@ -1965,6 +2062,7 @@ menu_button_key_press_cb(GtkWidget *widget,
 }
 
 
+/*ARGSUSED*/
 static void
 menu_pos_func(GtkMenu *menu, gint *x, gint *y,
               gboolean *push_in, gpointer user_data)
@@ -2054,7 +2152,7 @@ mb_proc(GtkAction *action)
         show_ascii_frame();
     } else if (EQUAL(name, "Memory")) {
         v->rstate = !v->rstate;
-        do_memory(v->rstate);
+        do_memory();
     } else if (EQUAL(name, "Contents")) {
         help_cb();
     } else if (EQUAL(name, "LSPlaces")) {
@@ -2068,10 +2166,10 @@ mb_proc(GtkAction *action)
     } else if (EQUAL(name, "ArithmeticPrecedence")) {
         toggle_expressions();
     }
-
 }
 
 
+/*ARGSUSED*/
 static void 
 mb_base_radio_proc(GtkAction *action, GtkRadioAction *current)
 {
@@ -2091,6 +2189,7 @@ mb_base_radio_proc(GtkAction *action, GtkRadioAction *current)
 }
 
 
+/*ARGSUSED*/
 static void
 mb_acc_radio_proc(GtkAction *action, GtkRadioAction *current)
 {
@@ -2100,10 +2199,15 @@ mb_acc_radio_proc(GtkAction *action, GtkRadioAction *current)
 	return;
     }
 
-    handle_menu_selection(X->mrec[(int) M_ACC], name[2]);
+    if (EQUAL(name, "SPOther")) {
+        show_precision_frame();
+    } else {
+        handle_menu_selection(X->mrec[(int) M_ACC], name[2]);
+    }
 }
 
 
+/*ARGSUSED*/
 static void
 mstz_proc(GtkAction *action)
 {
@@ -2116,7 +2220,7 @@ mstz_proc(GtkAction *action)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi), v->show_zeroes);
         v->doing_mi = 0;
 
-        show_display(v->MPdisp_val);
+	syntaxdep_show_display(); //show_display(v->MPdisp_val);
 	put_resource(R_ZEROES, set_bool(v->show_zeroes == TRUE));
 	make_registers();
     }
@@ -2182,6 +2286,7 @@ put_resource(enum res_type rtype, char *value)
 }
 
 
+/*ARGSUSED*/
 static void
 quit_cb(GtkWidget *widget, gpointer user_data)
 {
@@ -2212,6 +2317,59 @@ reset_mode_values(enum mode_type mtype)
 }
 
 
+/*ARGUSED*/
+static void
+spframe_adj_cb(GtkAdjustment *adj, gpointer data)
+{
+    g_print("New value is %f\n", adj->value);
+}
+
+
+/*ARGSUSED*/
+static void
+spframe_cancel_cb(GtkButton *button, gpointer user_data)
+{
+    gtk_widget_hide(X->spframe);
+}
+
+
+/*ARGSUSED*/
+static gboolean
+spframe_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+    if (event->keyval == GDK_minus) {
+        update_statusbar(_("Accuracy value out of range"),
+                         "gtk-dialog-error");
+        beep();
+    }
+
+    return(FALSE);
+}
+
+
+/*ARGSUSED*/
+static void
+spframe_ok_cb(GtkButton *button, gpointer user_data)
+{
+    char intval[5];
+    int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(X->spframe_val));
+
+    v->accuracy = val;
+
+    SPRINTF(intval, "%d", v->accuracy);
+    put_resource(R_ACCURACY, intval);
+
+    set_accuracy_menu_item(v->accuracy);
+    set_accuracy_tooltip(v->accuracy);
+    set_accuracy_toggle(v->accuracy);
+
+    make_registers();
+    refresh_display();
+
+    gtk_widget_hide(X->spframe);
+}
+
+
 static void
 set_button_state(GtkWidget *w, int isSensitive)
 {
@@ -2219,15 +2377,17 @@ set_button_state(GtkWidget *w, int isSensitive)
 }
 
 
-void
+static void
 set_accuracy_toggle(int val)
 {
     char name[MAXLINE];
     GtkWidget *acc;
 
-    SPRINTF(name, "/AccMenu/SP%c", val + '0');
-    acc = gtk_ui_manager_get_widget(X->ui, name);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(acc), TRUE);
+    if (val >= 0 && val <= 9) {
+        SPRINTF(name, "/AccMenu/SP%c", val + '0');
+        acc = gtk_ui_manager_get_widget(X->ui, name);
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(acc), TRUE);
+    }
 }
 
 
@@ -2245,8 +2405,10 @@ set_display(char *str, int minimize_changes)
     if (str == NULL || *str == 0) {
         str = " ";
     } else {
-	localize_number(localized, str);
+      if (!v->noparens) {
+      	localize_number(localized, str);
 	str = localized;
+      }
     }
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X->display_item));
     gtk_text_buffer_get_bounds(buffer, &start, &end);
@@ -2291,6 +2453,32 @@ set_display(char *str, int minimize_changes)
 }
 
 
+void
+write_display(char *str)
+{
+    char localized[MAX_LOCALIZED];
+    gchar *text;
+    GtkTextBuffer *buffer;
+    GtkTextIter start, end;
+
+    if (str == NULL || *str == 0) str = " ";
+
+    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X->display_item));
+    gtk_text_buffer_get_bounds(buffer, &start, &end);
+    text = gtk_text_buffer_get_text(buffer, &start, &end, TRUE);
+
+    gtk_text_buffer_delete(buffer, &start, &end);
+    
+    gtk_text_buffer_insert_with_tags_by_name(buffer,
+					     &end,
+					     str,
+					     -1,
+					     "x-large",
+					     NULL);
+    g_free(text);
+
+}
+
 #define SET_MENUBAR_ITEM_STATE(i, state) \
           g_object_set(gtk_ui_manager_get_action(X->ui, i),  \
 			"sensitive", state, NULL);
@@ -2433,7 +2621,7 @@ set_mode(enum mode_type mode)
 }
 
 
-void
+static void
 set_item(enum item_type itemtype, int val)
 {
     if (!v->started) {
@@ -2511,7 +2699,7 @@ set_show_zeroes_toggle(int state)
 }
 
 
-void
+static void
 show_ascii_frame()      /* Display ASCII popup. */
 {
     if (X->aframe == NULL) {
@@ -2526,6 +2714,22 @@ show_ascii_frame()      /* Display ASCII popup. */
 }
 
 
+static void
+show_precision_frame()      /* Display Set Precision popup. */
+{
+    if (X->spframe == NULL) {
+        create_spframe();
+    }
+
+    if (gdk_window_is_visible(X->spframe->window) == FALSE) {
+        ds_position_popup(X->kframe, X->spframe, DS_POPUP_LEFT);
+    }
+    gtk_window_set_focus(GTK_WINDOW(X->spframe), GTK_WIDGET(X->spframe_val));
+    gtk_widget_show(X->spframe);
+}
+
+
+/*ARGSUSED*/
 static void
 trig_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -2546,6 +2750,7 @@ start_tool()
 }
 
 
+/*ARGSUSED*/
 static void
 ts_proc(GtkAction *action)
 {
@@ -2555,7 +2760,7 @@ ts_proc(GtkAction *action)
 
     v->show_tsep = !v->show_tsep;
 
-    show_display(v->MPdisp_val);
+    syntaxdep_show_display();  //show_display(v->MPdisp_val);
     put_resource(R_TSEP, set_bool(v->show_tsep == TRUE));
     make_registers();
 }
